'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  var compressedClassesCache = { maxID: 0 };
  var cssSources = {};
  var aliases = {};

  function isCall(node) {
    return t.isCallExpression(node) && t.isIdentifier(node.callee, { name: 'css' }) && node.arguments.length === 1;
  }
  function isTagged(node) {
    return t.isTaggedTemplateExpression(node) && t.isIdentifier(node.tag, { name: 'css' });
  }

  function getInputAndTag(path) {
    if (isCall(path.node)) {
      var _path$get$0$evaluate = path.get('arguments')[0].evaluate();

      var confident = _path$get$0$evaluate.confident;
      var value = _path$get$0$evaluate.value;

      if (!confident || typeof value !== 'string') {
        throw new Error('Expected a constant string as the input to `insert-css-module`');
      }
      return { input: value, tag: path.node.callee, match: true };
    } else if (isTagged(path.node)) {
      var _path$get$evaluate = path.get('quasi').evaluate();

      var _confident = _path$get$evaluate.confident;
      var _value = _path$get$evaluate.value;

      if (!_confident || typeof _value !== 'string') {
        throw new Error('Expected a constant string as the input to `insert-css-module`');
      }
      return { input: _value, tag: path.node.tag, match: true };
    } else {
      return { input: null, tag: null, match: false };
    }
  }
  function visitVariableDeclaration(path, opts, filename) {
    if (t.isVariableDeclaration(path.node) && path.node.declarations.length === 1 && path.node.declarations[0].init) {
      var _getInputAndTag = getInputAndTag(path.get('declarations')[0].get('init'));

      var input = _getInputAndTag.input;
      var tag = _getInputAndTag.tag;
      var match = _getInputAndTag.match;

      if (!match) {
        return;
      }

      var processor = (0, _postcss2.default)([(0, _postcssModules2.default)({
        getJSON: function getJSON(cssFileName, json) {
          aliases[path.node.declarations[0].id.name] = json;
        },
        generateScopedName: function generateScopedName(name, filename, css) {
          filename = (0, _path.resolve)(filename);
          if (opts.optimised) {
            var key = (0, _stringHash2.default)(filename).toString(36) + '_' + (0, _stringHash2.default)(name).toString(36);
            var id = void 0;
            var cache = compressedClassesCache;
            if (opts.cache) {
              try {
                cache = JSON.parse((0, _fs.readFileSync)(opts.cache, 'utf8'));
              } catch (ex) {
                if (ex.code === 'ENOENT') cache = { maxID: 0 };else throw ex;
              }
            }
            if (key in cache) {
              id = cache[key];
            } else {
              id = cache.maxID++;
              cache[key] = id;
            }
            if (opts.cache) {
              (0, _fs.writeFileSync)(opts.cache, JSON.stringify(cache, null, '  '));
            }
            return '_' + (opts.optimised === true ? id : opts.optimised + '_' + id);
          }
          var i = css.indexOf('.' + name);
          var numLines = css.substr(0, i).split(/[\r\n]/).length;
          var hash = (0, _stringHash2.default)(css).toString(36);

          return '_' + name + '_' + hash + '_' + numLines;
        }
      })].concat(postPlugins()));

      var transformedCSS = processor.process(input, {
        from: filename,
        to: filename
      }).css;

      if (opts.extractCSS) {
        cssSources[opts.extractCSS][filename] += transformedCSS;
        path.remove();
      } else {
        path.replaceWith(t.expressionStatement(t.callExpression(tag, [t.stringLiteral(transformedCSS)])));
      }
    }
  }

  function visitExpressionStatement(path, opts, filename) {
    if (t.isExpressionStatement(path.node)) {
      var _getInputAndTag2 = getInputAndTag(path.get('expression'));

      var input = _getInputAndTag2.input;
      var tag = _getInputAndTag2.tag;
      var match = _getInputAndTag2.match;

      if (!match) return;

      var processor = (0, _postcss2.default)(postPlugins());

      var transformedCSS = processor.process(input, {
        from: filename,
        to: filename
      }).css;
      if (opts.extractCSS) {
        cssSources[opts.extractCSS][filename] += input;
        path.remove();
      } else if (transformedCSS !== input) {
        path.replaceWith(t.expressionStatement(t.callExpression(tag, [t.stringLiteral(transformedCSS)])));
      }
    }
  }

  return {
    visitor: {
      // NoOps to ensure correct ordering by babel

      TaggedTemplateExpression: function TaggedTemplateExpression() {},
      TemplateLiteral: function TemplateLiteral() {},

      Program: {
        enter: function enter(path) {
          if (this.opts.extractCSS) {
            var filename = (0, _path.resolve)(this.file.opts.filename);
            cssSources[this.opts.extractCSS] = cssSources[this.opts.extractCSS] || {};
            cssSources[this.opts.extractCSS][filename] = '';
          }
        },
        exit: function exit(path) {
          var _this = this;

          if (this.opts.extractCSS) {
            (0, _fs.writeFileSync)(this.opts.extractCSS, Object.keys(cssSources[this.opts.extractCSS]).sort().map(function (filename) {
              return cssSources[_this.opts.extractCSS][filename];
            }).join('\n'));
          }
        }
      },
      VariableDeclaration: function VariableDeclaration(path) {
        if (path.node._insertedCSS) return;
        var filename = (0, _path.resolve)(this.file.opts.filename);
        visitVariableDeclaration(path, this.opts, filename);
        if (path.node) path.node._insertedCSS = true;
      },
      ExpressionStatement: function ExpressionStatement(path) {
        if (path.node._insertedCSS) return;
        var filename = (0, _path.resolve)(this.file.opts.filename);
        visitExpressionStatement(path, this.opts, filename);
        if (path.node) path.node._insertedCSS = true;
      },
      CallExpression: function CallExpression(path) {
        if (path.node._insertedCSS) return;
        if (t.isIdentifier(path.node.callee) && path.node.callee.name in aliases && path.node.arguments.length === 1) {
          var _path$get$0$evaluate2 = path.get('arguments')[0].evaluate();

          var confident = _path$get$0$evaluate2.confident;
          var value = _path$get$0$evaluate2.value;

          if (!confident || typeof value !== 'string') {
            throw new Error('Expected a constant string as the input to `insert-css-module`');
          }
          if (!(value in aliases[path.node.callee.name])) {
            throw new Error('Unrecognised class name "' + value + '"');
          }
          path.replaceWith(t.stringLiteral(aliases[path.node.callee.name][value]));
        }
        if (path.node) path.node._insertedCSS = true;
      }
    }
  };
};

var _fs = require('fs');

var _path = require('path');

var _stringHash = require('string-hash');

var _stringHash2 = _interopRequireDefault(_stringHash);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _postcssModules = require('postcss-modules');

var _postcssModules2 = _interopRequireDefault(_postcssModules);

var _core = require('cssnano/dist/lib/core');

var _core2 = _interopRequireDefault(_core);

var _postcssDiscardComments = require('postcss-discard-comments');

var _postcssDiscardComments2 = _interopRequireDefault(_postcssDiscardComments);

var _postcssMinifyGradients = require('postcss-minify-gradients');

var _postcssMinifyGradients2 = _interopRequireDefault(_postcssMinifyGradients);

var _postcssReduceTransforms = require('postcss-reduce-transforms');

var _postcssReduceTransforms2 = _interopRequireDefault(_postcssReduceTransforms);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _postcssConvertValues = require('postcss-convert-values');

var _postcssConvertValues2 = _interopRequireDefault(_postcssConvertValues);

var _postcssCalc = require('postcss-calc');

var _postcssCalc2 = _interopRequireDefault(_postcssCalc);

var _postcssColormin = require('postcss-colormin');

var _postcssColormin2 = _interopRequireDefault(_postcssColormin);

var _postcssOrderedValues = require('postcss-ordered-values');

var _postcssOrderedValues2 = _interopRequireDefault(_postcssOrderedValues);

var _postcssMinifySelectors = require('postcss-minify-selectors');

var _postcssMinifySelectors2 = _interopRequireDefault(_postcssMinifySelectors);

var _postcssMinifyParams = require('postcss-minify-params');

var _postcssMinifyParams2 = _interopRequireDefault(_postcssMinifyParams);

var _postcssNormalizeCharset = require('postcss-normalize-charset');

var _postcssNormalizeCharset2 = _interopRequireDefault(_postcssNormalizeCharset);

var _postcssMinifyFontValues = require('postcss-minify-font-values');

var _postcssMinifyFontValues2 = _interopRequireDefault(_postcssMinifyFontValues);

var _postcssDiscardUnused = require('postcss-discard-unused');

var _postcssDiscardUnused2 = _interopRequireDefault(_postcssDiscardUnused);

var _postcssNormalizeUrl = require('postcss-normalize-url');

var _postcssNormalizeUrl2 = _interopRequireDefault(_postcssNormalizeUrl);

var _postcssMergeIdents = require('postcss-merge-idents');

var _postcssMergeIdents2 = _interopRequireDefault(_postcssMergeIdents);

var _postcssMergeLonghand = require('postcss-merge-longhand');

var _postcssMergeLonghand2 = _interopRequireDefault(_postcssMergeLonghand);

var _postcssDiscardDuplicates = require('postcss-discard-duplicates');

var _postcssDiscardDuplicates2 = _interopRequireDefault(_postcssDiscardDuplicates);

var _postcssMergeRules = require('postcss-merge-rules');

var _postcssMergeRules2 = _interopRequireDefault(_postcssMergeRules);

var _postcssDiscardEmpty = require('postcss-discard-empty');

var _postcssDiscardEmpty2 = _interopRequireDefault(_postcssDiscardEmpty);

var _postcssUniqueSelectors = require('postcss-unique-selectors');

var _postcssUniqueSelectors2 = _interopRequireDefault(_postcssUniqueSelectors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function postPlugins() {
  var plugins = [];

  plugins.push((0, _postcssDiscardComments2.default)());
  plugins.push((0, _postcssMinifyGradients2.default)());
  plugins.push((0, _postcssReduceTransforms2.default)());
  // see https://github.com/ai/browserslist#queries
  plugins.push((0, _autoprefixer2.default)({ browsers: ['last 3 versions'], cascade: false }));
  plugins.push((0, _postcssConvertValues2.default)({ length: false }));
  plugins.push((0, _postcssCalc2.default)());
  plugins.push((0, _postcssColormin2.default)());
  plugins.push((0, _postcssOrderedValues2.default)());
  plugins.push((0, _postcssMinifySelectors2.default)());
  plugins.push((0, _postcssMinifyParams2.default)());
  plugins.push((0, _postcssNormalizeCharset2.default)({ add: false }));
  // minify-font-values should be run before discard-unused
  plugins.push((0, _postcssMinifyFontValues2.default)());
  plugins.push((0, _postcssDiscardUnused2.default)());
  plugins.push(_postcssNormalizeUrl2.default);
  plugins.push((0, _core2.default)());
  // Optimisations after this are sensitive to previous optimisations in
  // the pipe, such as whitespace normalising/selector re-ordering
  plugins.push((0, _postcssMergeIdents2.default)());
  plugins.push((0, _postcssMergeLonghand2.default)());
  plugins.push((0, _postcssDiscardDuplicates2.default)());
  plugins.push((0, _postcssMergeRules2.default)());
  plugins.push((0, _postcssDiscardEmpty2.default)());
  plugins.push((0, _postcssUniqueSelectors2.default)());

  return plugins;
}